v9-0002-Add-assert-to-ensure-that-page-locks-don-t-partic.patch
application/octet-stream
Filename: v9-0002-Add-assert-to-ensure-that-page-locks-don-t-partic.patch
Type: application/octet-stream
Part: 0
Patch
Same data as JSON:
GET /api/v1/attachments/:id/patch
the parsed metadata as JSON — format, series position, per-file stats; never the diff bytes.
API reference →
Format: format-patch
Series: patch v9-0002
Subject: Add assert to ensure that page locks don't participate in deadlock cycle.
| File | + | − |
|---|---|---|
| src/backend/storage/lmgr/lock.c | 23 | 2 |
From 74874bad13296ce298f66f334ac7c9cf41af0204 Mon Sep 17 00:00:00 2001
From: Dilip Kumar <dilip.kumar@enterprisedb.com>
Date: Sun, 15 Mar 2020 21:21:07 +0530
Subject: [PATCH v9 2/4] Add assert to ensure that page locks don't participate
in deadlock cycle.
Assert that we don't acquire any other heavyweight lock while holding the
page lock except for relation extension. However, these locks are never
taken in reverse order which implies that page locks will never
participate in the deadlock cycle.
Similar to relation extension, page locks are also held for a short
duration, so imposing such a restriction won't hurt.
Author: Dilip Kumar, with few changes by Amit Kapila
Reviewed-by: Amit Kapila and Kuntal Ghosh
Discussion: https://postgr.es/m/CAD21AoCmT3cFQUN4aVvzy5chw7DuzXrJCbrjTU05B+Ss=Gn1LA@mail.gmail.com
---
src/backend/storage/lmgr/lock.c | 25 +++++++++++++++++++++++--
1 file changed, 23 insertions(+), 2 deletions(-)
diff --git a/src/backend/storage/lmgr/lock.c b/src/backend/storage/lmgr/lock.c
index 076f436..57db6cc 100644
--- a/src/backend/storage/lmgr/lock.c
+++ b/src/backend/storage/lmgr/lock.c
@@ -185,6 +185,18 @@ static int FastPathLocalUseCount = 0;
*/
static bool IsRelationExtensionLockHeld = false;
+/*
+ * Flag to indicate if the page lock is held by this backend. We don't
+ * acquire any other heavyweight lock while holding the page lock except for
+ * relation extension. However, these locks are never taken in reverse order
+ * which implies that page locks will also never participate in the deadlock
+ * cycle.
+ *
+ * Similar to relation extension, page locks are also held for a short
+ * duration, so imposing such a restriction won't hurt.
+ */
+static bool IsPageLockHeld = false;
+
/* Macros for manipulating proc->fpLockBits */
#define FAST_PATH_BITS_PER_SLOT 3
#define FAST_PATH_LOCKNUMBER_OFFSET 1
@@ -863,6 +875,13 @@ LockAcquireExtended(const LOCKTAG *locktag,
Assert(!IsRelationExtensionLockHeld);
/*
+ * We don't acquire any other heavyweight lock while holding the page lock
+ * except for relation extension lock.
+ */
+ Assert(!IsPageLockHeld ||
+ (locktag->locktag_type == LOCKTAG_RELATION_EXTEND));
+
+ /*
* Prepare to emit a WAL record if acquisition of this lock needs to be
* replayed in a standby server.
*
@@ -1310,10 +1329,10 @@ SetupLockInTable(LockMethod lockMethodTable, PGPROC *proc,
}
/*
* Check and set/reset the flag that we hold the relation extension lock.
*
* It is callers responsibility that this function is called after
* acquiring/releasing the relation extension lock.
*
* Pass acquired = true if lock is acquired, false otherwise.
*/
@@ -1322,6 +1341,8 @@ CheckAndSetLockHeld(LOCALLOCK *locallock, bool acquired)
{
if (LOCALLOCK_LOCKTAG(*locallock) == LOCKTAG_RELATION_EXTEND)
IsRelationExtensionLockHeld = acquired;
+ else if (LOCALLOCK_LOCKTAG(*locallock) == LOCKTAG_PAGE)
+ IsPageLockHeld = acquired;
}
/*
@@ -1310,10 +1329,10 @@ SetupLockInTable(LockMethod lockMethodTable, PGPROC *proc,
}
/*
- * Check and set/reset the flag that we hold the relation extension lock.
+ * Check and set/reset the flag that we hold the relation extension/page lock.
*
* It is callers responsibility that this function is called after
@@ -1310,10 +1329,10 @@ SetupLockInTable(LockMethod lockMethodTable, PGPROC *proc,
*
* It is callers responsibility that this function is called after
- * acquiring/releasing the relation extension lock.
+ * acquiring/releasing the relation extension/page lock.
*
* Pass acquired = true if lock is acquired, false otherwise.
--
1.8.3.1